//+------------------------------------------------------------------+
//|                                                        DinTF.mq4 |
//|                                  Copyright 2023, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
#property indicator_chart_window

#include <Canvas\iCanvas_CB.mqh> //https://www.mql5.com/ru/code/22164

int totalSteps;
MqlRates rt[];

int N_tick;
int _t=0;
int MaxBar=0;
datetime SFD=0;
int nn=0;
double K=1;
bool overflow=false;
datetime t_start;
bool rdraw = false;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnInit() {
   MaxBar=TerminalInfoInteger(TERMINAL_MAXBARS);
   SFD=(datetime)SeriesInfoInteger(_Symbol,PERIOD_M1,SERIES_SERVER_FIRSTDATE);
   //ChartSetInteger(0,CHART_EVENT_MOUSE_WHEEL,1);
   Canvas.CurentFont("Calibri Light",30,30);

   ulong t0=GetMicrosecondCount();
   _t=int(ResSynchr(rt));
   t0=GetMicrosecondCount()-t0;
   Print("load - "+IntegerToString(t0));
   if (_t==0) {
      //ChartSetInteger(0,CHART_SHOW,false);
      ChartSetInteger(0,CHART_KEYBOARD_CONTROL,false);
      ChartSetInteger(0,CHART_CROSSHAIR_TOOL,false);
      ChartSetInteger(0,CHART_MOUSE_SCROLL,false);
      EventSetTimer(2);
      Canvas.Comm("Please wait, loading data...");
      if (MaxBar!=100000000) Canvas.Comm("You need to set the number of max bars in the chart - Unlimited!");
      Canvas.Update();
   } else {
      EventKillTimer();
      totalSteps=int(W.Width);
      t_start=rt[0].time;
      K=pow(10, log10(_t/W.Width)/totalSteps);
      if(K==0) K=1.01;
      Screen("1 Bar = "+DoubleToString((PeriodSeconds()/60),2)+" min ",PeriodSeconds());
   }
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnDeinit(const int reason) {
   if (reason!=REASON_CHARTCHANGE) {
      //ChartSetInteger(0,CHART_SHOW,true);
      ChartSetInteger(0,CHART_CROSSHAIR_TOOL,true);
      ChartSetInteger(0,CHART_MOUSE_SCROLL,true);
      ChartSetInteger(0,CHART_KEYBOARD_CONTROL,true);
      ArrayFree(rt);
      ResourceFree("::MqlRates"+ IntegerToString(ChartGetInteger(0,CHART_WINDOW_HANDLE)));
   }
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]) {
   return(rates_total);
}
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer() {
   static bool load = true;
   if (load) {
      if(_t<=0) {
         totalSteps=int(W.Width);
         _t=CopyRates(_Symbol,PERIOD_M1,0,MaxBar,rt); 
         if (_t>0) t_start=rt[0].time;
         else return;
         K=pow(10, log10(_t/W.Width)/totalSteps);
         if(K==0) K=1.01;
      } else {
         ulong t0=GetMicrosecondCount();
         ResSynchr(rt);
         t0=GetMicrosecondCount()-t0;
         Print("save - "+IntegerToString(t0));
         EventSetMillisecondTimer(30);
         load = false;
         Screen("1 Bar = "+DoubleToString((PeriodSeconds()/60),2)+" min ",PeriodSeconds());
         Print("Used memory = " + string(ArraySize(rt)*sizeof(MqlRates)) + " bytes");
      }
   } else if (rdraw) {
      int pos=W.Width-W.MouseX;
      if (pos>W.Width) pos=totalSteps;
      if(pos<0) pos=0;
      double pp=60*pow(K,pos);
      Screen("1 Bar = "+DoubleToString((pp/60),2)+" minutes ",(int)pp);
      rdraw = false;
     }
}
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam) {
   if(id==CHARTEVENT_MOUSE_MOVE || id==CHARTEVENT_CHART_CHANGE) {
      static int lastMX = 0; 
      static int lastW = W.Width; 
      if (lastW!=W.Width) {
         totalSteps=W.Width;
         t_start=rt[0].time;
         K=pow(10, log10(_t/W.Width)/totalSteps);
         if(K==0) K=1.01;
         lastW=W.Width;
      }
      if (W.MouseX!=lastMX) {
         rdraw = true;
         lastMX = W.MouseX;
      }
   }
}
//+------------------------------------------------------------------+
void Screen(string str="",int v=86400) {
   ulong t=GetMicrosecondCount();
   if (_t<W.Width) return;
   nn++;
   if(_t==0)
      return;
   double hi[];
   double lo[];
   datetime ti[];
   int k=0;
   int n=W.Width;
   ArrayResize(hi,n);
   ArrayResize(lo,n);
   ArrayResize(ti,n);
   int a=_t-1;
   while(k<n && a>=0) {
      double Hi=rt[a].high;
      double Lo=rt[a].low;
      int tim=int(rt[a].time-rt[a].time%v);
      if (tim<t_start) {
         overflow=true;
         break;
      } else overflow=false;
      a--;
      while(a>=0 && rt[a].time-rt[a].time%v==tim) {
         if(Hi<rt[a].high)
            Hi=rt[a].high;
         if(Lo>rt[a].low)
            Lo=rt[a].low;
         a--;
      }
      hi[k]=Hi;
      lo[k]=Lo;
      ti[k]=tim;
      k++;
   }
   //if (k<n) k++;
   int I=n-1;
   if(I<0)
      I=0;
   double max=hi[I];
   double min=lo[I];
   for(int i=k-1; i>=0; i--) {
      if(max<hi[i])
         max=hi[i];
      if(min>lo[i])
         min= lo[i];
   }
   double d=(max-min)/(W.Height-20);
   if(d==0)
      d=1;
   double C=200;
   double gr=200.0/W.Height;
   for(int i=0; i<W.Height; i++) {
      Canvas.LineHorizontal(0,W.Width-1,i,ARGB(255,0,Round(C)/4,Round(C)));
      C=C-gr;
   }
   for(int i1=I,j=0; i1>=0; i1--,j++) {
      int i=(i1>=k)?k-1:i1;
      if (i>=n) i=n-1;
      if (i>=ArraySize(hi)) break;
      int y=Round(W.Height-(hi[i]-min)/d-10);
      Canvas.LineVertical(j,Round(W.Height-(lo[i]-min)/d-10),y,0xFF8080FF);
      Canvas.LineVertical(j,0,y-1,0xFF001020);
   }
   if(str!="") {
      Canvas.TextColor=0x80A0A020;
      Canvas.TextPosition(5.0,5.0);
      Canvas.CurentFont("Calibri Light",30,40,0xFF8000AA);
      Canvas.TextColor=0xFFFFFFFF;
      Canvas.Comm(str+" , Framing time: "+DoubleToString(((GetMicrosecondCount()-t)/1000),1)+ " mc");
   }
   //Canvas.LineVertical(W.MouseX,0,W.Height-1,0x700000FF);
   int i=W.Width-W.MouseX;
   if (i<0) i=0;
   if(i>=k) i=k-1;
   int y_=Round(((W.Height-(hi[i]-min)/d-10)+W.Height-(lo[i]-min)/d-10)/2);
   Canvas.Circle(W.MouseX,y_,6,0xFF00FFFF);
   if (W.MouseX<W.Width-150) Canvas.TextPosition(W.MouseX+10,y_-40);
   else Canvas.TextPosition(W.MouseX-160,y_-40);
   Canvas.FontSizeSet(15);
   Canvas.CurentFont("Tahoma",15,17,0xFFFFFFFF);
   Canvas.Comm("High = "+DoubleToString(hi[i],_Digits));
   Canvas.Comm("Low  = "+DoubleToString(lo[i],_Digits));
   Canvas.Comm("Time  = "+TimeToString(ti[i]));
   Canvas.Update();
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
uint ResSynchr(MqlRates &r[]) {
   const uint su=sizeof(MqlRates)/sizeof(uint);
   union RatesUint {
      MqlRates rt;
      uint u[sizeof(MqlRates)/sizeof(uint)];
   };
   int size=ArraySize(r);
   uint arr[];
   RatesUint v;
   string Name="::MqlRates"+ IntegerToString(ChartGetInteger(0,CHART_WINDOW_HANDLE));
   if (size>0) {
      uint Size=su*size+1;
      ArrayResize(arr,Size);
      arr[0]= size;
      uint j=1;
      for (int i=0; i<size; i++,j+=su) {
         v.rt=r[i];
         ArrayCopy(arr,v.u,j,0);
      }
      if(!ResourceCreate(Name,arr,Size,1,0,0,0,COLOR_FORMAT_XRGB_NOALPHA)) printf("Error create Resource: "+DoubleToString(GetLastError(),0));
      else return size;
   } else {
      uint w;
      uint h;
      if(ResourceReadImage(Name,arr,w,h)) {
         uint j=1;
         ArrayResize(r,arr[0]);
         for (int i=0; i<int(arr[0]); i++,j+=su) {
            ArrayCopy(v.u,arr,0,j,su);
            r[i]=v.rt;
         }
         return arr[0];
      }
   }
   return 0;
}
//+------------------------------------------------------------------+
